unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ComCtrls, ExtCtrls, StdCtrls, LCLTMSFNCListBox, LCLTMSFNCMemo, LCLTMSFNCPageControl, LCLTMSFNCHTMLText,
  LCLTMSFNCImage, LCLTMSFNCBitmapContainer;

type

  { TForm1 }

  TForm1 = class(TForm)
    Memo1: TMemo;
    pc: TPageControl;
    tmsi: TTMSFNCImage;
    tmsht: TTMSFNCHTMLText;
    tsi: TTabSheet;
    tstp: TTabSheet;
    tsip: TTabSheet;
    tmsbc: TTMSFNCBitmapContainer;
    tmslb: TTMSFNCListBox;
    procedure FormCreate(Sender: TObject);
    procedure tmslbDblClick(Sender: TObject);
  private

  public
    procedure UpdateFormDir(aDir: String);
    procedure AddItemWI(aFileName: String; IsFolder: Boolean);
    procedure GetInfo(aFileName: String; IsFolder: Boolean);
    function ParentDir(aPath: String): String;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

uses
  FileUtil;

var
  current_dir: String;

procedure TForm1.FormCreate(Sender: TObject);
begin
  current_dir := ExtractFilePath(Application.ExeName);
  Caption := current_dir;
  tmslb.BitmapContainer := tmsbc;
  tmsht.Text := '';
  tstp.TabVisible := False;
  tsip.TabVisible := False;
  //add images to container
  tmsbc.AddFromFile('up.png', 'up');
  tmsbc.AddFromFile('folder.png', 'folder');
  tmsbc.AddFromFile('file.png', 'file');
  tmsbc.AddFromFile('text.png', 'text');
  tmsbc.AddFromFile('arch.png', 'arch');
  tmsbc.AddFromFile('exec.png', 'exec');
  tmsbc.AddFromFile('image.png', 'image');
  tmsbc.AddFromFile('music.png', 'music');
  tmsbc.AddFromFile('video.png', 'video');
  //
  UpdateFormDir(current_dir);
  GetInfo(current_dir, True);
  pc.ActivePage := tsi;
end;

procedure TForm1.tmslbDblClick(Sender: TObject);
var
  t: Integer = 0;
  newfn: String;
begin
  pc.ActivePage := tsi;
  tstp.TabVisible := False;
  tsip.TabVisible := False;                                                     
  if tmslb.ItemIndex >= 0 then
    t := tmslb.Items[tmslb.ItemIndex].Tag;    
  if t = 0 then
    exit;
  if t = 100 then
    newfn := ParentDir(current_dir)
  else
    newfn := IncludeTrailingPathDelimiter(current_dir) + tmslb.Items[tmslb.ItemIndex].Text;
  GetInfo(newfn, (t div 100) < 3);
  if (t = 100) or (t = 200) then
  begin
    current_dir := newfn;
    Caption := current_dir;
    UpdateFormDir(current_dir);
  end
  else
  begin
    if t = 301 then
    begin
      tstp.TabVisible := True;
      Memo1.Lines.LoadFromFile(newfn);
      pc.ActivePage := tstp;
    end
    else if t = 302 then
    begin
      tsip.TabVisible := True;
      tmsi.Bitmap.LoadFromFile(newfn);
      tmsi.Stretch := (tmsi.Bitmap.Width > tmsi.Width) or (tmsi.Bitmap.Height > tmsi.Height);
      pc.ActivePage := tsip;
    end;
  end;
end;

procedure TForm1.UpdateFormDir(aDir: String);
var
  sl: TStringList;
  i: Integer;
begin
  tmslb.Items.BeginUpdate;
  tmslb.Items.Clear;
  sl := TStringList.Create;
  AddItemWI('..', True);  //level up
  FindAllDirectories(sl, aDir, False);
  for i := 0 to sl.Count - 1 do
    AddItemWI(sl[i], True); //dirs
  sl.Clear;
  FindAllFIles(sl, aDir, '*', False);
  for i := 0 to sl.Count - 1 do
    AddItemWI(sl[i], False); //files
  sl.Free;
  tmslb.Items.EndUpdate;
end;

procedure TForm1.AddItemWI(aFileName: String; IsFolder: Boolean);
var
  lbi: TTMSFNCListBoxItem;
  bn, s, e: String;
  t: Integer;
begin
  if AFileName = '..' then
    s := aFileName
  else
    s := ExtractFileName(aFileName);
  if s = '..' then
  begin
    bn := 'up';
    t := 100;
  end
  else if IsFolder then
  begin
    bn := 'folder';
    t := 200;
  end
  else
  begin
    t := 300;
    e := LowerCase(ExtractFileExt(s));
    case e of
      '.txt', '.xml', '.json', '.sql', '.pas', '.lfm', '.lpr', '.pp', '.dpr', '.cs', '.html', '.php':
        begin
          bn := 'text';
          t := 301;
        end;
      '.exe', '.msi': bn := 'exec';
      '.zip', '.7z', '.rar', '.tar', '.bz', '.gz', '.tgz': bn := 'arch';
      '.bmp', '.gif', '.jpg', '.jpeg', '.png', '.webp', '.tif', '.tiff', '.svg':
        begin
          bn := 'image';
          t := 302;
        end;
      '.mp3', '.wav', '.flac', '.ogg': bn := 'music';
      '.avi', '.mkv', '.mpg', '.mpeg', '.mp4', '.flv', '.wmv', '.rmvb': bn := 'video';
      else
        bn := 'file';
    end;
  end;
  lbi := tmslb.AddItem(s);
  lbi.BitmapName := bn;
  lbi.Tag := t;
end;

procedure TForm1.GetInfo(aFileName: String; IsFolder: Boolean);
begin
  if IsFolder then
    tmsht.Text := Format('<b>Type:</b> Directory<br><b>Path:</b> %s', [aFileName])
  else
  begin
    tmsht.Text := Format('<b>Type:</b> File<br><b>Name:</b> %s<br><b>Path:</b> %s<br><b>Size:</b> %s<br><b>Date:</b> %s',
      [ExtractFileName(aFileName), ExtractFilePath(aFileName), FormatFloat('# ### ### ##0', FileSize(aFileName)),
      FormatDateTime('yyyy-mm-dd hh:nn:ss', FileDateToDateTime(FileAge(aFileName)))]);
  end;
end;

function TForm1.ParentDir(aPath: String): String;
var
  sarr: TStringArray;
  i: Integer;
begin
  Result := '';
  sarr := aPath.Split(['/', '\'], TStringSplitOptions.ExcludeEmpty);
  if Length(sarr) < 2 then
    Result := aPath
  else
  begin
    for i := Low(sarr) to High(sarr) - 1 do
      Result := Result + IncludeTrailingPathDelimiter(sarr[i]);
  end;
end;

end.

